/*
	Agrupamos las funciones principales bajo una función que se ejecuta nada más cargar el script.
*/

$(function(){
	//Definimos las constantes y variables necesarias para guardar y recuperar las notas.
	var STORAGE_KEY = 'entries_list',
		SEL_CLASS = 'sel';
	
	//Flag para saber si estamos en un dispositivo móvil (smartphone o tablet) ejecutando Cordova/PhoneGap.
	var mobile_app = false;

	//Variables para saber si el elemento de la lista está desplegado y para generar el sistema que convierte las coordenadas a dirección (reverse GeoCoder).
	var collapse = true,
		location = null,
		geocoder = null;
		
	//Referencias a los elementos HTML del documento
	var container = $('section.entries'),
		titleField = $('#title'),
		contentField = $('#content'),
		latField = $('#lat'),
		lngField = $('#lng'),
		newEntry = $('#newEntry'),
		storeEntry = $('#storeEntry'),
		noEntries = $('#noEntries');

	//Variables para guardar la longitud y la latitud de los elementos a desplegar en el mapa.
	var latDisplay = '',
		lngDisplay = '';
	
	//Comprobamos que podemos acceder a LocalStorage, en caso contratio, le mostramos una alerta al usuario.
	if (localStorage) {
			// Inicializamos la aplicación
			loadEntries();
			assignEvents();
		} else {
			sweetAlert("Oops...", "This App requires LocalStorage and your system doesn't support it", "error");
		}
		
	
	if (navigator.geolocation) {
		console.log("Localización activada");
	}
	else{
		console.log("Localización no activada");
	}
	
	
	/**
	 * Con este método comprobamos si estamos ejecutando PhoneGap en un terminal con iOS o Android.
	 */
	function isPhoneGap() {
		return (cordova || PhoneGap || phonegap) && /^file:\/{3}[^\/]/i.test(window.location.href) && /ios|iphone|ipod|ipad|android/i.test(navigator.userAgent);
	}
	
	if (isPhoneGap()) {
		//Estamos ejecutando con Cordova/PhoneGap.
		mobile_app = true;
	} else {
		//No estamos ejecutando con Cordova/PhoneGap.
		//Cargamos el GeoCoder.
		onDeviceReady();
	}
	
	/**
	 * Con este método comprobamos si se han cargado todas las librerías de Cordova, con esto evitamos que en iOS salga el aviso en el que se solicita el permiso de lcoalziación cada vez que entramos.
	 */
	$(function(){
  document.addEventListener("deviceready", onDeviceReady, false);
	})

	function onDeviceReady() {
  		geocoder = new google.maps.Geocoder();
		//Listener para saber si ha cambiado la posición del usuario.
		if (navigator.geolocation) {
			navigator.geolocation.watchPosition(
				function (pos) {
					//Convertimos la longitud y la latitud a una dirección y se la mostramos al usuario.
					geolocatePosition(pos, function(strPosition) {
						location = strPosition;
						var locationPlaceHolder = $('#location');
						locationPlaceHolder.html('Near <a href="#"> ' + strPosition + '</a>');
						locationPlaceHolder.show();
					});
				},
				function (err) {
					//Error, lo mostramos por consola.
					console.log(err);
				}
			)
		}
}

function onSuccess(position) {
  // your callback here 
}

function onError(error) { 
  // your callback here
}
  
  
	
	


	/**
	 * Método para borrar entradas de la lista. Se muestra un SweetAlert para que el usuario confirme si está seguro. En caso afirmativo, se borra.
	 */
	function deleteEntry() {
		
		var article = $(this).closest('article');	
			swal({   
					title: "Are you sure?",   
					text: "You will not be able to recover this GeoNote!",   
					type: "warning",   
					showCancelButton: true,   
					confirmButtonColor: "#DD6B55",   
					confirmButtonText: "Yes, delete it!",   
					closeOnConfirm: false }, 
						function(){  
							//Borrar entrada de la lista.
							var entries = entriesList();
							delete entries[article.attr('rel')];
							article.fadeOut('fast', function() {
								$(this).remove();
	
								//Guardamos la nueva lista.
								saveEntries(entries);
							}); 
							swal("Deleted!", "Your GeoNote has been deleted.", "success"); 	
					});
	}

	
	/**
	 * Método para asignar funciones a los distintos iconos (borrar, editar y mostrar el mapa).
	 */
	function assignEventsToEntries() {
		// Añadimos el evento para borrar al pulsar el icono.
		$("article p img.delete").click(deleteEntry);
		
		// Añadimos el evento para mostrar el mapa al pulsar el icono.
		$("article p img.mapicon").click(function() {
			//Buscamos la entrada seleccionada
			var entries = entriesList();
			var key = $(this).closest('article').attr('rel');
			var entry = entries[key];
	
			//Preparamos latDisplay, lngDisplay para mostrar el mapa.
			latDisplay = entry.lat;
			lngDisplay = entry.lng;
			
			//Añadimos el título y la descripción a la pantalla modal			
			document.getElementById("myModalLabel").innerHTML = entry.title;
			document.getElementById("myModalText").innerHTML = entry.content;
			
			//Mostramos el Google Map con un marker en la localización guardada
			var latlng = new google.maps.LatLng(parseFloat(latDisplay), 
				  					parseFloat(lngDisplay));
			var myOptions = {
					zoom: 15,
					center: latlng,
					mapTypeControl: false,
					navigationControlOptions: {style: google.maps.NavigationControlStyle.SMALL},
					mapTypeId: google.maps.MapTypeId.ROADMAP
			};
			var map = new google.maps.Map(document.getElementById("mapcanvas"), myOptions);
				  
				  //Añadimos el marker.
			var marker = new google.maps.Marker({
				position: latlng, 
				map: map, 
				title:"My location"
			});
				  
			google.maps.event.addListenerOnce(map, 'idle', function() { 
			google.maps.event.trigger(map, 'resize');
			map.panTo(latlng);
			});

			
		
		//Mostramos la pantalla modal.
		$('#geolocation').modal('show');
			
		});
		
		// Añadimos el evento para editar al pulsar el icono.
		$("article p img.edit").click(function() {
			//Buscamos la entrada seleccionada
			var entries = entriesList();
			var key = $(this).closest('article').attr('rel');
			var entry = entries[key];

			//Rellenamos los campos a editar.
			titleField.val(entry.title);
			contentField.val(entry.content);

			//Mostramos el formulario.
			showForm(key);
		});

		//Añadimos un callback para detectar un Swype en una fila.
		var swipeCallback = function(event) {
			var srcElement = $(event.srcElement) || $(event.target);
			// Añadimos/quitamos la imagen para borrar una entrada.
			if (srcElement.find('img').length == 0) {
				//Quitamos la imagen al pulsar en otro elemento.
				container.find('h2 .swipe-delete').remove();

				srcElement.prepend(function() {
					return $('<span><img class="swipe-delete" src="img/i_delete.png" style="height:20px;width:20px;" /></span>').click(deleteEntry);
				});
			} else {
				srcElement.find('img').remove();
			}
		}

		//Añadimos un callback para detectar cuando se pulsa el título de una entrada.
		var clickCallback = function(event, target) {
			target = $(target);

			//Comprobamos si el elemento seleccionado es un H2
			if (target.get(0).tagName == 'H2') {
				//Hacemos que sólo el elemento actual esté desplegado.
				var isSelected = target.parent().hasClass(SEL_CLASS);
				$('article').removeClass(SEL_CLASS);
				if (!isSelected) {
					target.parent().addClass(SEL_CLASS);
				}
			}
				
			if (!target.hasClass('swipe-delete')) {
				//Quitamos el icono de Swype si se hace click en otra función
				container.find('h2 .swipe-delete').remove();
			}
		}

		//Capturamos el Swype
		var swipeOptions = {
			click:clickCallback,
			swipeLeft:swipeCallback,
			swipeRight:swipeCallback,
			threshold:50
		};
		$("article h2").swipe(swipeOptions);
	}

	/**
	 * En caso de que se hayan producido errores al no rellenar todos los campos del formulario, los quitamos cuando se produce una nueva acción.
	 */
	function resetForm() {
		// Reseteamos todos los campos
		titleField.val('').removeClass('error');
		contentField.val('').removeClass('error');
	}

	/**
	 * Este método se encarga de procesar el formulario.
	 */
	function submitForm(event) {
		event.preventDefault();

		var type = $(this).attr('rel');

		//Validamos que se hayan rellenado todos los campos
		var error = false;
		titleField.removeClass('error');
		contentField.removeClass('error');

		//En caso de que se no se hayan rellenado todos, los marcamos.
		if (titleField.val().length == 0) {
			titleField.addClass('error');
			error = true;
		}
		if (contentField.val().length == 0) {
			contentField.addClass('error');
			error = true;
		}

		//Mostramos una alerta en caso de no haber rellenado todo para alertar al usuario.
		if (error) {
			sweetAlert("Oops...", "Please fill in all the fields!", "error");
			return;
		}

		//Creamos una nueva entrada de JSON con nuestro modelo de datos. Guardamos el título, el contenido, la latitud, la longitud y la localización devuelta por el GeoCoder.
		var json_entry = {'title': titleField.val(),
							'content': contentField.val(),
							'lat': latField.val(),
							'lng': lngField.val(),
							'location': location};

		var entries = entriesList();
		
		//Si estamos editando, quitamos el elemento anterior.
		if (type != 'new') {
			delete entries[type];
			$('article[rel='+type+']').remove();
		}

		//Usamos como llave primaria el momento de creación (timestamp).
		var key = Math.round(new Date().getTime() / 1000);

		//Refrescamos las entradas.
		var entry_html = htmlForEntry(key, json_entry);
		container.prepend(entry_html);
		
		//Guardamos la entrada.
		entries[key] = json_entry;
		saveEntries(entries);

		//Reasignamos eventos.
		assignEventsToEntries();

		//Mostramos la lista de entradas.
		showList();

		//Reseteamos el formulario.
		resetForm();
	}

	/**
	 * Este método recarga las entradas si se ha producido un cambio en LocalStorage.
	 */
	function storageChanged(e) {
		loadEntries();
	}

	/**
	 * Este método recarga las entradas cambiadas.
	 */
	function entriesChanged() {
		// Mostramos un mensaje si no hay entradas, si las hay lo quitamos.
		noEntries.remove();
    	if (container.html().length == 0) {
    		container.after('<p id="noEntries">You have no entries yet</p>');
    		noEntries = $('#noEntries');
    	}
	}

	/**
	 * Asignamos los distintos elementos al formulario de edición.
	 */
	function assignEvents() {
		$('h1').click(function() {
			//Expandimos o colpsamos todos los elementos.
			if (collapse) {
				$('article').addClass(SEL_CLASS);
			} else {
				$('article').removeClass(SEL_CLASS);
			}
			collapse = !collapse;
		});

		//Añadimos el link para abrir el form.
		newEntry.click(function() {
			showForm('new');
		});

		//Link para volver a la lista
		$('#btnCancel').click(function() {
			if ((titleField.val().length != 0) || (contentField.val().length != 0)){
				swal({   
					title: "Are you sure?",   
					text: "You have not saved this changes!",   
					type: "warning",   
					showCancelButton: true,   
					confirmButtonColor: "#DD6B55",   
					confirmButtonText: "Yes, discard them!",   
					closeOnConfirm: false }, 
						function(){  
							//Borramos la entrada de la lista
							resetForm();
							showList();
							swal("Discarded!", "Your changes has been discarded.", "success"); 	
						}); 
			}
			else{
				resetForm();
				showList();
			}
						
	});
	

		//Quitamos el icono de borrar abierto con Swype al pulsar en otros elementos.
		$(document.body).click(function(event) {
			var srcElement =  event.srcElement || event.target;
			if (srcElement.tagName !='H2' && !$(srcElement).hasClass('swipe-delete')) {
				container.find('h2 .swipe-delete').remove();
			}
		});

		//Enviamos el formulario.
		storeEntry.submit(submitForm);

		//Listener para saber si han cambiado los contenidos de la memoria.
		if ('storage' in navigator) {
			window.addEventListener('storage', storageChanged, false); 
		}

		
	}
	
	/**
	 * Función para obtener la posición actual del usuario.
	 */
	function geolocatePosition(position, callback) {
		if (geocoder) {
			
			//añadimos la longitud y la latitud a los campos ocultos del fomulario para luego poder recuperarlos.
			document.getElementById("lat").value = position.coords.latitude;
			document.getElementById("lng").value = position.coords.longitude;
			
			
			//Obtenemos la dirección usando la latitud y la longitud.
			var latlng =  new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
			geocoder.geocode({'latLng': latlng}, function (results, status) {
				if (status == google.maps.GeocoderStatus.OK) {
					callback(results[0].formatted_address);
				}
			});
		} 
	}

	/**
	 * Función para guardar la entrada.
	 */
	function saveEntries(entries) {
		localStorage.setItem(STORAGE_KEY, JSON.stringify(entries));

		//Procesamos los cambios.
		entriesChanged();
	}
	
	/**
	 * Función para mostrar el form.
	 */
	function showForm(type) {
		//Mostramos el form.
		storeEntry.attr('rel',type);
		container.hide();
		newEntry.hide();
		noEntries.hide();
		storeEntry.fadeIn('fast');
	}

	/**
	 * Función para mostrar la lista de entradas.
	 */
	function showList() {
		newEntry.show();
		noEntries.show();		
		storeEntry.hide();
		container.fadeIn('fast');
	}

	/**
	 * Función para generar la lista de entradas.
	 */
	function entriesList() {
		var entries_str = localStorage.getItem(STORAGE_KEY);
    	if (!entries_str) {
    		//Si no hay nada guardado mostramos el mensaje por defecto.
    		var entries = {};
    		saveEntries(entries);
    		return entries;
    	}
    	return JSON.parse(entries_str);
	}

	/**
	 * Función para generar el desplegable de una entrada en la lista.
	 */
	function htmlForEntry(key, entry) {
		//Generamos el HTML que va a mostrar el elemento.
		var location = '';
		var googMapsAddr = '';
		if (entry.location) {
			location = '<a class="map" href="#">'+entry.location+'</a>';
			googMapsAddr = 'http://maps.google.es?q='+entry.location;
			googMapsAddr = googMapsAddr.split(' ').join('+');
		}
		var actionLinks = '<span class="actions"><img class="edit" src="img/i_edit.png" /><img class="delete" src="img/i_delete.png" /><img class="mapicon" src="img/i_map.png" /></span>';
		var date = new Date(key * 1000);
		
		return '<article rel="'+key+'"><h2>'+entry.title+'</h2><p>'+actionLinks+entry.content+'<span class="time">'+date.toLocaleString()+'</span>'+location+'</p></article>';
	}

	/**
	 * Función para cargar las entradas.
	 */
	function loadEntries() {
		//Vaciamos la lista.
    	container.empty();

		//Si hay entradas las mostramos todas.
		var entries = entriesList();
    	for (var key in entries) {
    		container.prepend(htmlForEntry(key, entries[key]));
    	}

		//Procesamos los cambios.
		entriesChanged();

    	//Asignamos los eventos necesarios.
    	assignEventsToEntries();
	}
});